/* 
 * Copyright (C) 2013 DENSO CORPORATION
 * Copyright (c) 2013 BMW Car IT GmbH
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 * 
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef IVI_HMI_CONTROLLER_SERVER_PROTOCOL_H
#define IVI_HMI_CONTROLLER_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct ivi_hmi_controller;

extern const struct wl_interface ivi_hmi_controller_interface;

#ifndef IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM
#define IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM
enum ivi_hmi_controller_layout_mode {
	IVI_HMI_CONTROLLER_LAYOUT_MODE_TILING = 0,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_SIDE_BY_SIDE = 1,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_FULL_SCREEN = 2,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_RANDOM = 3,
};
#endif /* IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM */

#ifndef IVI_HMI_CONTROLLER_HOME_ENUM
#define IVI_HMI_CONTROLLER_HOME_ENUM
enum ivi_hmi_controller_home {
	IVI_HMI_CONTROLLER_HOME_OFF = 0,
	IVI_HMI_CONTROLLER_HOME_ON = 1,
};
#endif /* IVI_HMI_CONTROLLER_HOME_ENUM */

/**
 * ivi_hmi_controller - set up and control IVI style UI
 * @UI_ready: inform the ready for drawing desktop.
 * @workspace_control: start controlling a surface by server
 * @switch_mode: request mode switch of application layout
 * @home: request displaying/undisplaying home screen
 *
 * 
 */
struct ivi_hmi_controller_interface {
	/**
	 * UI_ready - inform the ready for drawing desktop.
	 *
	 * 
	 */
	void (*UI_ready)(struct wl_client *client,
			 struct wl_resource *resource);
	/**
	 * workspace_control - start controlling a surface by server
	 * @seat: (none)
	 * @serial: (none)
	 *
	 * Reference protocol to control a surface by server. To control
	 * a surface by server, it gives seat to the server to e.g. control
	 * Home screen. Home screen has several workspaces to group
	 * launchers of wayland application. These workspaces are drawn on
	 * a horizontally long surface to be controlled by motion of input
	 * device. E.g. A motion from right to left happens, the viewport
	 * of surface is controlled in the ivi-shell by using ivi-layout.
	 * client can recognizes the end of controlling by event
	 * "workspace_end_control".
	 */
	void (*workspace_control)(struct wl_client *client,
				  struct wl_resource *resource,
				  struct wl_resource *seat,
				  uint32_t serial);
	/**
	 * switch_mode - request mode switch of application layout
	 * @layout_mode: (none)
	 *
	 * hmi-controller loaded to ivi-shall implements 4 types of
	 * layout as a reference; tiling, side by side, full_screen, and
	 * random.
	 */
	void (*switch_mode)(struct wl_client *client,
			    struct wl_resource *resource,
			    uint32_t layout_mode);
	/**
	 * home - request displaying/undisplaying home screen
	 * @home: (none)
	 *
	 * home screen is a reference implementation of launcher to
	 * launch wayland applications. The home screen has several
	 * workspaces to group wayland applications. By defining the
	 * following keys in weston.ini, user can add launcher icon to
	 * launch a wayland application to a workspace. [ivi-launcher]
	 * workspace-id=0 : id of workspace to add a launcher icon-id=4001
	 * : ivi id of ivi_surface to draw a icon
	 * icon=/home/user/review/build-ivi-shell/data/icon_ivi_flower.png
	 * : path to icon image
	 * path=/home/user/review/build-ivi-shell/weston-dnd : path to
	 * wayland application
	 */
	void (*home)(struct wl_client *client,
		     struct wl_resource *resource,
		     uint32_t home);
};

#define IVI_HMI_CONTROLLER_WORKSPACE_END_CONTROL	0

#define IVI_HMI_CONTROLLER_WORKSPACE_END_CONTROL_SINCE_VERSION	1

static inline void
ivi_hmi_controller_send_workspace_end_control(struct wl_resource *resource_, int32_t is_controlled)
{
	wl_resource_post_event(resource_, IVI_HMI_CONTROLLER_WORKSPACE_END_CONTROL, is_controlled);
}

#ifdef  __cplusplus
}
#endif

#endif
